// This software code is made available "AS IS" without warranties of any 
// kind.  You may copy, display, modify and redistribute the software
// code either by itself or as incorporated into your code; provided that
// you do not remove any proprietary notices.  Your use of this software
// code is at your own risk and you waive any claim against Amazon Web 
// Services LLC or its affiliates with respect to your use of this software 
// code. (c) Amazon Web Services LLC or its affiliates.

using System;
using System.Collections.Generic;
using System.Text;
using System.Security.Policy;
using System.Web;
using System.Net;
using System.IO;
using System.Collections; 

namespace AmazonWebServices
{
    /// <summary>
    /// This class implements all the operations of EC2 services. For a complete 
    /// documentation about the various operations and parameters, please refer 
    /// the EC2 Developer Guide 
    /// </summary>
   public class EC2Operation
    {
        private string _AWSAccessKeyId;
        private string _AWSSecretAccessKey;

        private const string DEFAULT_VERSION = "2007-01-19";  
        private const string DEFAULT_SIGNATURE_VERSION = "1";
        private const string DEFAULT_HOST = "ec2.amazonaws.com";
        //private const string DEFAULT_PROTOCOL = "https";
        private const string DEFAULT_PROTOCOL = "http";
        

        private string _Version;
        private string _SignatureVersion;
        private String _Host;
        private String _Protocol;

       
        /// <summary>
        /// Constructor - sets the values for AWSAccessKeyId and AWSSecretAccessKey
        /// </summary>
        /// <param name="AWSAccessKeyId"></param>
        /// <param name="AWSSecretAccessKey"></param>
        public EC2Operation(string accessKeyId, string secretAccessKey)
        : this(accessKeyId, secretAccessKey, DEFAULT_VERSION, DEFAULT_SIGNATURE_VERSION, DEFAULT_HOST, DEFAULT_PROTOCOL) {}

        public EC2Operation(string accessKeyId, string secretAccessKey, String version)
        : this(accessKeyId, secretAccessKey, version, DEFAULT_SIGNATURE_VERSION, DEFAULT_HOST, DEFAULT_PROTOCOL) {}

        public EC2Operation(string accessKeyId, string secretAccessKey, String version, String sigVersion)
        : this(accessKeyId, secretAccessKey, version, sigVersion, DEFAULT_HOST, DEFAULT_PROTOCOL) {}

        public EC2Operation(string accessKeyId, string secretAccessKey, String version, String sigVersion, String host)
        : this(accessKeyId, secretAccessKey, version, sigVersion, host, DEFAULT_PROTOCOL) {}

        public EC2Operation(string accessKeyId, string secretAccessKey, String version, String sigVersion, String host, String protocol)
        {
            //Console.WriteLine(secretAccessKey);
            _Version = DEFAULT_VERSION;
            _SignatureVersion = DEFAULT_SIGNATURE_VERSION;
            _Host = host;
            _Protocol = protocol;
            _AWSAccessKeyId = accessKeyId;
            _AWSSecretAccessKey = secretAccessKey;
        }
        
        // Factory method for creating EC2Request objects
        private EC2Request CreateEC2Request()
        {
            return new EC2Request(_AWSAccessKeyId, _AWSSecretAccessKey, _SignatureVersion, _Version, _Host, _Protocol);
        }
        
        
        #region Images : DescribeImages, RegisterImages, DeregisterImages 
        
        /// <summary>
        /// The DescribeImages operation returns information about AMIs available for use by the user. This
        /// includes both public AMIs (those available for any user to launch) and private AMIs (those owned by
        /// the user making the request and those owned by other users that the user making the request has explicit
        /// Amazon EC2 launch permissions for).
        /// 
        /// If no parametres are provided, then it will return all the images 
        /// </summary>
        /// <param name="imageId">Image description</param>
        /// <param name="owner">Owner of AMI to describe</param>
        /// <param name="executableBy">Describe AMI that the specified user has launch permission for</param>
        /// <returns></returns>
        public EC2Response DescribeImages(IList<string> imageIds, IList<string> owners, IList<string> executableBy)
        {
            EC2Request req = CreateEC2Request();
            req.AddParameters("ImageId", imageIds);
            req.AddParameters("Onwer", owners);
            req.AddParameters("ExecutableBy", executableBy);
            return req.Execute("DescribeImages");
        }
        

        /// <summary>
        /// The RegisterImage operation registers an AMI with Amazon EC2. Images must be registered before
        /// they can be launched.
        /// 
        /// Each AMI is associated with an unique ID which is provided by the EC2 service via the Registerimage
        /// operation. As part of the registration process, Amazon EC2 will retrieve the specified image manifest
        /// from Amazon S3 and verify that the image is owned by the user requesting image registration.
        /// 
        /// The image manifest is retrieved once and stored within the Amazon EC2 network. Any modifications to
        /// an image in Amazon S3 invalidate this registration. If you do have to make changes and upload a new
        /// image deregister the previous image and register the new image.
        /// </summary>
        /// <param name="imageLocation">Full path to your AMI manifest in Amazon S3 storage.</param>
        /// <returns></returns>
        public EC2Response RegisterImage(string imageLocation)
        {
            if (imageLocation == null || imageLocation.Length == 0)
                throw new ArgumentNullException("Image Location can't be null or empty"); 
                
            EC2Request req = CreateEC2Request();
            req.AddParameter("ImageLocation", imageLocation);
            return req.Execute("RegisterImage");
        }

        /// <summary>
        /// The DeregisterImage operation deregisters an AMI. Once deregistered, instances of the AMI may no
        /// longer be launched.
        /// 
        /// If no Image Id is provided, this method will throw an excpetion 
        /// </summary>
        /// <param name="imageId">Unique ID of a machine image,</param>
        /// <returns></returns>
        public EC2Response DeregisterImage(string imageId)
        {
            if (imageId == null || imageId.Length == 0)
                throw new ArgumentNullException("Image Id can't be null or empty");

            EC2Request req = CreateEC2Request();
            req.AddParameter("ImageId", imageId);
            return req.Execute("DeregisterImage");
        }
        
        #endregion 




        #region Instances : DescribeInstances, RunInstances, TerminateInstances, RebootInstances 

        /// <summary>
        /// Operation : DescribeInstances
        /// If no InstanceId is provided, then this method will return all the InstanceIds 
        /// </summary>
        /// <param name="instanceId"></param>
        /// <returns></returns>
        public EC2Response DescribeInstances(string instanceId)
        {
            EC2Request req = CreateEC2Request();
            req.AddParameter("InstanceId", instanceId);
            return req.Execute("DescribeInstances");
        }
        
              
        
        /// <summary>
        /// The RunInstances operation launches a specified number of instances.
        /// </summary>
        /// <param name="imageId">Id of the AMI to launch instances based on.</param>
        /// <param name="minCount">Minimum number of instances to launch.</param>
        /// <param name="maxCount">Maximum number of instances to launch.</param>
        /// <param name="keyName">Name of the keypair to launch instances with.</param>
        /// <param name="userData">The user data available to the launched instances. This should be base64-encoded.</param>
        /// <param name="securityGroup">The addressing scheme to launch the instance with. 
        ///  The addressing type can be direct or public. In the direct scheme the instance has one IP address that is not NATted. For
        /// </param>
        /// <returns></returns>
        public EC2Response RunInstances(string imageId, 
            int minCount, 
            int maxCount, 
            string keyName, 
            string userData, 
            string securityGroup) 
        {
            EC2Request req = CreateEC2Request();
            req.AddParameter("ImageId", imageId);
            req.AddParameter("MinCount", minCount.ToString());
            req.AddParameter("MaxCount", maxCount.ToString());
            req.AddParameter("KeyName", keyName);
            req.AddParameter("UserData", userData);
            req.AddParameter("SecurityGroup", securityGroup);
                       

            return req.Execute("RunInstances");

        }


       

        /// <summary>
        /// The TerminateInstances operation shuts down the Instance ID passed to it
        /// </summary>
        /// <param name="instanceId">Instance ID</param>
        /// <returns></returns>
        public EC2Response TerminateInstances(string instanceId)
        {

            if (instanceId == null || instanceId.Length == 0)
            {
                throw new ArgumentNullException("Instance ID can't be null or empty");                     
            }


            EC2Request req = CreateEC2Request();
            req.AddParameter("InstanceId", instanceId);
            return req.Execute("TerminateInstances");
        }

        

        /// <summary>
        /// The RebootInstances operation requests a reboot the instance as specified by the InstanceId. 
        /// This operation is asynchronous; it only queues a request to reboot the specified instance. 
        /// The operation will succeed provided the instance is valid and belongs to the user. 
        /// Terminated instances will be ignored.
        /// </summary>
        /// <param name="instanceId">InstanceId</param>
        /// <returns></returns>
        public EC2Response RebootInstances(string instanceId)
        {

            if (instanceId == null || instanceId.Length == 0)
                throw new ArgumentNullException("InstanceId can't be null or empty"); 


            EC2Request req = CreateEC2Request();
            req.AddParameter("InstanceId", instanceId);
            return req.Execute("RebootInstances");                        

        }

        #endregion 


        

        #region Keypairs : CreateKeyPair, DescribeKeyPairs, DeleteKeypair 

        /// <summary>
        /// The CreateKeyPair operation creates a new 2048 bit RSA keypair and returns a unique ID that can be 
        /// used to reference this keypair when launching new instances.
        /// </summary>
        /// <param name="keyName">A unique name for this key.</param>
        /// <returns></returns>
        public EC2Response CreateKeyPair(string keyName)
        {
            if (keyName == null || keyName.Length == 0)
                throw new ArgumentNullException("KeyName can't be null or empty");


            EC2Request req = CreateEC2Request();
            req.AddParameter("KeyName", keyName);
            return req.Execute("CreateKeyPair");                  

        }


        /// <summary>
        /// Operation : DeleteKeyPair
        /// This method deletes the particular key-pair that is passed; if a null 
        /// or empty key-pair is passed, it will throw an excpetion 
        /// </summary>
        /// <param name="keyName"></param>
        /// <returns></returns>
        public EC2Response DeleteKeyPair(string keyName)
        {
            if (keyName == null || keyName.Length == 0)
                throw new ArgumentNullException("KeyName can't be null or empty");

            EC2Request req = CreateEC2Request();
            req.AddParameter("KeyName", keyName);
            return req.Execute("DeleteKeyPair");    

            
        }



        /// <summary>
        /// Operation : DescribeKeyPairs
        /// If no keyName is provided, then it will return all the available keyPairs 
        /// </summary>
        /// <param name="keyName"></param>
        /// <returns>XML string value of the response</returns>
        public EC2Response DescribeKeyPairs(string keyName)
        {            
            EC2Request req = CreateEC2Request();
            req.AddParameter("KeyName", keyName);
            return req.Execute("DescribeKeyPairs");                       
        }



        #endregion 



                
        #region Image Attributes : ModifyImageAttribute, DescribeImageAttribute, ResetImageAttribute 

        /// <summary>
        /// The ModifyImageAttribute operation modifies an attribute of an AMI.
        /// 
        /// Currently the only attribute supported is launchPermission. By modifying this attribute it 
        /// is possible to make an AMI public or to grant specific users launch permissions for the AMI. 
        /// To make the AMI public add the group=all attribute item. To grant launch permissions for a specific
        /// user add a userId=<userid> attribute item.
        /// </summary>
        /// 
        /// <param name="imageId">AMI Id to modify an attribute on.</param>
        /// <param name="attribute">Specifies the attribute to modify.</param>
        /// <param name="operationType">Specifies the operation to perform on the attribute.</param>
        /// <param name="userId">User id to add to or remove from the launchPermission attribute.</param>
        /// <param name="userGroup">User group to add to or remove from the launchPermission attribute.</param>
        /// <returns></returns>
        public EC2Response ModifyImageAttribute(string imageId, 
            string attribute,
            string operationType,
            string userId,
            string userGroup
            )
        {

            if (imageId == null || imageId.Length == 0)
                throw new ArgumentNullException("ImageId can't be null or empty");

            if (attribute == null || attribute.Length == 0)
                throw new ArgumentNullException("Attribute can't be null or empty");

            if (operationType == null || operationType.Length == 0)
                throw new ArgumentNullException("OperationType can't be null or empty"); 

            EC2Request req = CreateEC2Request();
            req.AddParameter("ImageId", imageId);
            req.AddParameter("Attribute", attribute);
            req.AddParameter("OperationType", operationType);
            req.AddParameter("UserId", userId);
            req.AddParameter("UserGroup", userGroup);

            return req.Execute("ModifyImageAttribute");                       

        }





        /// <summary>
        /// The DescribeImageAttribute operation returns information about an attribute of an AMI.
        /// </summary>
        /// 
        /// <param name="imageId">Id of the AMI for which an attribute will be described.</param>
        /// <param name="attribute">Specifies the attribute to describe.</param>
        /// <returns></returns>
        public EC2Response DescribeImageAttribute(string imageId, string attribute)             
        {

            if (imageId == null || imageId.Length == 0)
                throw new ArgumentNullException("ImageId can't be null or empty");

            if (attribute == null || attribute.Length == 0)
                throw new ArgumentNullException("Attribute can't be null or empty");                       


            EC2Request req = CreateEC2Request();
            req.AddParameter("ImageId", imageId);
            req.AddParameter("Attribute", attribute);
            
            return req.Execute("DescribeImageAttribute");                

        }



        /// <summary>
        /// The ResetImageAttribute operation resets an attribute of an AMI to its default value.
        /// </summary>
        /// 
        /// <param name="imageId">Id of the AMI for which an attribute will be described.</param>
        /// <param name="attribute">Specifies the attribute to describe.</param>
        /// <returns></returns>
        public EC2Response ResetImageAttribute(string imageId, string attribute)
        {

            if (imageId == null || imageId.Length == 0)
                throw new ArgumentNullException("ImageId can't be null or empty");

            if (attribute == null || attribute.Length == 0)
                throw new ArgumentNullException("Attribute can't be null or empty");

            EC2Request req = CreateEC2Request();
            req.AddParameter("ImageId", imageId);
            req.AddParameter("Attribute", attribute);
            
            return req.Execute("ResetImageAttribute");              

        }

        #endregion




        #region Security Groups : AuthorizeSecurityGroupIngress, RevokeSecurityGroupIngress, CreateSecurityGroup, DescribeSecurityGroups, DeleteSecurityGroup 


        /// <summary>
        /// The AuthorizeSecurityGroupIngress operation adds permissions to a security group.
        /// </summary>
        /// <param name="groupName">Name of the group to modify.</param>
        /// <param name="sourceSecurityGroupName">Name of security group to authorize access to when operating on a user/group pair.</param>
        /// <param name="sourceSecurityGroupOwnerId">Owner of security group to authorize access to when operating on a user/group pair.</param>
        /// <param name="ipProtocol">IP protocol to authorize access to when operating on a CIDR IP.</param>
        /// <param name="fromPort">Bottom of port range to authorize access to when operating on a CIDR IP.</param>
        /// <param name="toPort">Top of port range to authorize access to when operating on a CIDR IP.</param>
        /// <param name="cidrIp">CIDR IP range to authorize access to when operating on a CIDR IP.</param>
        /// <returns></returns>
        public EC2Response AuthorizeSecurityGroupIngress(string groupName,
            string sourceSecurityGroupName,
            string sourceSecurityGroupOwnerId,
            string ipProtocol, 
            string fromPort,
            string toPort,
            string cidrIp
            )
        {
            if (groupName == null || groupName.Length == 0)
                throw new ArgumentNullException("GroupName can't be null or empty");
            
            EC2Request req = CreateEC2Request();
            req.AddParameter("GroupName", groupName);
            req.AddParameter("SourceSecurityGroupName", sourceSecurityGroupName);
            req.AddParameter("SourceSecurityGroupOwnerId", sourceSecurityGroupOwnerId);
            req.AddParameter("IpProtocol", ipProtocol);
            req.AddParameter("FromPort", fromPort);
            req.AddParameter("ToPort", toPort);
            req.AddParameter("CidrIp", cidrIp);
                         
            return req.Execute("AuthorizeSecurityGroupIngress");        
                                    
        }




        /// <summary>
        /// The RevokeSecurityGroupIngress operation revokes existing permissions that were previously granted to a security 
        /// group. The permissions to revoke must be specified using the same values originally used to grant the permission.  
        /// </summary>
        /// <param name="groupName">Name of the group to modify.</param>
        /// <param name="sourceSecurityGroupName">Name of security group to authorize access to when operating on a user/group pair.</param>
        /// <param name="sourceSecurityGroupOwnerId">Owner of security group to authorize access to when operating on a user/group pair.</param>
        /// <param name="ipProtocol">IP protocol to authorize access to when operating on a CIDR IP.</param>
        /// <param name="fromPort">Bottom of port range to authorize access to when operating on a CIDR IP.</param>
        /// <param name="toPort">Top of port range to authorize access to when operating on a CIDR IP.</param>
        /// <param name="cidrIp">CIDR IP range to authorize access to when operating on a CIDR IP.</param>
        /// <returns></returns>
        public EC2Response RevokeSecurityGroupIngress(string groupName,
            string sourceSecurityGroupName,
            string sourceSecurityGroupOwnerId,
            string ipProtocol,
            string fromPort,
            string toPort,
            string cidrIp
            )
        {
            if (groupName == null || groupName.Length == 0)
                throw new ArgumentNullException("GroupName can't be null or empty");


            EC2Request req = CreateEC2Request();
            req.AddParameter("GroupName", groupName);
            req.AddParameter("SourceSecurityGroupName", sourceSecurityGroupName);
            req.AddParameter("SourceSecurityGroupOwnerId", sourceSecurityGroupOwnerId);
            req.AddParameter("IpProtocol", ipProtocol);
            req.AddParameter("FromPort", fromPort);
            req.AddParameter("ToPort", toPort);
            req.AddParameter("CidrIp", cidrIp);

            return req.Execute("RevokeSecurityGroupIngress");            

        }




        /// <summary>
        /// The CreateSecurityGroup operation creates a new security group.
        /// </summary>
        /// <param name="groupName">Name for the new security group.</param>
        /// <param name="groupDescription">Description of the new security group.</param>
        /// <returns></returns>
        public EC2Response CreateSecurityGroup(string groupName, string groupDescription)
        {
            if (groupName == null || groupName.Length == 0)
                throw new ArgumentNullException("GroupName can't be null or empty");

            if (groupDescription == null || groupDescription.Length == 0)
                throw new ArgumentNullException("GroupDescription can't be null or empty");


            EC2Request req = CreateEC2Request();
            req.AddParameter("GroupName", groupName);
            req.AddParameter("GroupDescription", groupDescription);           

            return req.Execute("CreateSecurityGroup");     

        }




        /// <summary>
        /// The DescribeSecurityGroups operation returns information about security group
        /// owned by the user making the request.
        /// 
        /// If no security group names are provided, information of all security groups will 
        /// be returned. 
        /// 
        /// If a group is specified that does not exist a fault is returned.
        /// </summary>
        /// <param name="groupName">Security group</param>
        /// <returns></returns>
        public EC2Response DescribeSecurityGroups(string groupName)
        {
            EC2Request req = CreateEC2Request();
            req.AddParameter("GroupName", groupName);

            return req.Execute("DescribeSecurityGroups");     
        }




        /// <summary>
        /// The DeleteSecurityGroup operation deletes a security group.
        /// 
        /// If an attempt is made to delete a security group and any instances exist that 
        /// are members of that group a fault is returned.
        /// </summary>
        /// <param name="groupName">Name of the security group to delete.</param>
        /// <returns></returns>
        public EC2Response DeleteSecurityGroup(string groupName)
        {
            if (groupName == null || groupName.Length == 0)
                throw new ArgumentNullException("GroupName can't be null or empty");

            EC2Request req = CreateEC2Request();
            req.AddParameter("GroupName", groupName);

            return req.Execute("DeleteSecurityGroup");    

        }

        #endregion
        



        #region GetConsoleOutput 


        /// <summary>
        /// The GetConsoleOutput operation retrieves console output that has been posted for the 
        /// specified instance.
        /// 
        /// Instance console output is buffered and posted shortly after instance boot, reboot 
        /// and once the instance is terminated. Only the most recent 64 KB of posted output is 
        /// available. Console output is available for at least 1 hour after the most recent post.
        /// </summary>
        /// 
        /// <param name="instanceId">InstanceId</param>
        /// <returns></returns>
        public EC2Response GetConsoleOutput(string instanceId)
        {
            if (instanceId == null || instanceId.Length == 0)
                throw new ArgumentNullException("InstanceId can't be null or empty");


            EC2Request req = CreateEC2Request();
            req.AddParameter("InstanceId", instanceId);
            return req.Execute("GetConsoleOutput");
            
        }

        
        
        #endregion
         
    }
}
